//
// Smooth.hpp:
//	Interface definition for classes Channel and smooth.
//
// Purpose:
//	Provide functions to "smooth" the incoming data from
// the glove.  'smooth' is derived from cfGlove, whose data is
// unfiltered.  'Channel' provides different buffering variables for
// the different input variables X, Y, Z and Rotation.
//
// You can instantiate either "cfGlove" or "smooth", the difference
// being that you will have filtered results with deglitch and
// unfiltered results with cfGlove.  You might want raw data for glove
// hardware testing or calibration functions.
//
// Author(s): Mark Pflaging   C-serve  70233,1552
// Copyright 1992   Mark Thomas Pflaging
//
// Also see authorships in OOPGLOVE.DOC.
// Date:    5/15/92 - 6/4/92
//
#ifndef __SMOOTH_HPP
#define __SMOOTH_HPP

#ifndef __CF_GLOVE_HPP
#include  "cf_Glove.hpp"
#endif

#ifndef __INI_HPP
#include "ini.hpp"
#endif

class smooth;

class Channel {
	int x1, x2, lax, lcx, lsx;
	const int ACCEL, EXTEND;
	int ox;			// last x for hysterisis.
	const int HYST;
	int good_x;
	const int BOUND;
public:
	Channel(int maxAccel, int extendTime,
	   int hystQuanta = 0, int maxPixelJump = 0) :
		x1(0), x2(0), lax(0), lcx(0), ox(-1000),
		ACCEL(maxAccel), EXTEND(extendTime),
		BOUND(maxPixelJump), HYST(hystQuanta) {}
	void zero() { good_x = x1 = x2 = lax = lcx = lsx = 0; }
	void deGlitch(smooth & rawData, GloveData::dimensions which);
	void hysteresis(smooth & rawData, GloveData::dimensions which);
	void keepInBounds(smooth & rawData, GloveData::dimensions which);
};

class smooth : public cfGlove {
	Channel *data[RotChann + 1];

	void deGlitch(GloveData::dimensions which);
	void hysteresis(GloveData::dimensions which);
	void keepInBounds(GloveData::dimensions which);
	static char * title;
public:
	smooth(InitFile & ini) : cfGlove(ini) {
		data[XChann] = new Channel(
			ini.find(title, "XmaxAccel",     8),
			ini.find(title, "XextendTime",   3),
			ini.find(title, "XhystQuanta",   2),
			ini.find(title, "XmaxPixelJump", 50)
		);
		data[YChann] = new Channel(
			ini.find(title, "YmaxAccel",     8),
			ini.find(title, "YextendTime",   3),
			ini.find(title, "YhystQuanta",   2),
			ini.find(title, "YmaxPixelJump", 50)
		);
		data[ZChann] = new Channel(
			ini.find(title, "ZmaxAccel",     8),
			ini.find(title, "ZextendTime",   3),
			ini.find(title, "ZhystQuanta",   0),
			ini.find(title, "ZmaxPixelJump", 0)
		);
		data[RotChann] = new Channel(
			ini.find(title, "RotMaxAccel",     2),
			ini.find(title, "RotExtendTime",   3),
			ini.find(title, "RotHystQuanta",   0),
			ini.find(title, "RotMaxPixelJump", 0)
		);
	}

	int updateGloveData();
	void perform() {
		/* keepInBounds(YChann); */
		deGlitch(RotChann);
		hysteresis(RotChann);
	}
};

#endif
